//
//  ViewController.swift
//  CalculatorDemo
//
//  Created by Kevin McNeish on 12/10/14.
//  Copyright (c) 2014 Oak Leaf Enterprises, Inc. All rights reserved.
//

import UIKit

class ViewController: UIViewController {
    
    enum Operation: Int {
        
        case None = 0, Add, Subtract, Multiply, Divide,
        Equals, Clear, PositiveNegative, Percent
    }
    
    @IBOutlet weak var lblTotal: UILabel!
    @IBOutlet var operationHighlightImages: [UIImageView]!
    
    
    // The display value
    var value = ""
    // Previous value
    var previousValue = 0.0
    // Specifies if the last key pressed was an operation
    var isLastActionOperation = true
    // Previous operation key pressed
    var previousOperation = Operation.None
    // Previous operation actually performed
    var previousOperationPerformed = Operation.None
    // Number formatter
    var formatter = NSNumberFormatter()
    
    let maxDigits = 9
    let maxFractionDigits = 8
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        // Initialize variables
        self.formatter.numberStyle = NSNumberFormatterStyle.DecimalStyle
        self.formatter.maximumFractionDigits = maxFractionDigits
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    // MARK: - Calculator User Interface
    @IBAction func highlightOperation(sender: UIButton) {
        
        for imageView in self.operationHighlightImages {
            
            imageView.alpha = (imageView.tag == sender.tag) ? 1 : 0
        }
    }
    
    func hideOperationHighlight() {
        for imageView in self.operationHighlightImages {
            imageView.alpha = 0
        }
    }
    
    @IBAction func numberTouched(sender: UIButton) {
        
        var numberString = ""
        var tagNumber = sender.tag
        numberString = (tagNumber == -1) ? "." : "\(tagNumber)"
        self.hideOperationHighlight()
        
        self.lblTotal.text = self.processNumber(numberString)
    }
    
    @IBAction func operationTouched(sender: UIButton) {
        
        let operation = Operation(rawValue: sender.tag)
        
        self.lblTotal.text = self.performOperation(operation!)
    }
    
    // MARK: - Calculator Core Logic
    
    func processNumber(character: String) -> String {
        
        // Clear the display value if the last key was an operation
        if self.isLastActionOperation {
            self.value = ""
        }
        
        // Make sure we haven't reached maximum digits
        // First, remove all commas, then check length
        var tempString = self.value.stringByReplacingOccurrencesOfString(",", withString: "")
        
        if tempString.rangeOfString(".") == nil {
            if count(tempString) == maxDigits {
                return self.value
            }
        }
        else {
            if count(tempString) == maxDigits + 1 {
                return self.value
            }
        }
        
        // Decimal point processing
        if (character == ".") {
            
            // Don't allow multiple decimal points
            if self.value.rangeOfString(".") != nil {
                return self.value
            }
            
            // Prepend a zero if the display is empty
            // and the user entered a decimal point
            if count(self.value) == 0 {
                self.value = "0"
            }
            
            // Append the decimal point to the current string value
            self.value += character
        }
        else {
            // Remove all commas
            self.value = self.value.stringByReplacingOccurrencesOfString(",", withString: "")
            
            // Append the specified character to the current value
            self.value += character
            
            // Convert string to an NSNumber, adding commas in formatting
            let number = self.formatter.numberFromString(self.value)
            
            // Convert back to a string
            self.value = self.formatter.stringFromNumber(number!)!
        }
        
        self.isLastActionOperation = false
        
        return self.value
    }
    
    // Performs the specified operation, if it's an operation
    // that should be performed immediately, otherwise,
    // performs any previous operation
    func performOperation(operation: Operation) -> String {
        
        // convert the display value from string to double
        var currentValue = (self.value.stringByReplacingOccurrencesOfString(",", withString: "") as NSString).doubleValue
        
        let doOperation = (self.self.previousOperation == Operation.Equals) ? self.previousOperationPerformed : self.previousOperation
        
        switch operation {
            
            // Check first for operations that should be
            // performed immediately
        case .Clear:
            self.clear()
            
        case .PositiveNegative:
            self.makePositiveNegative()
            
        case .Percent:
            
            currentValue = self.applyPercentage(currentValue)
            var number = NSNumber(double: currentValue)
            self.value = formatter.stringFromNumber(number)!
            self.isLastActionOperation = false
            return self.value
            
        default:
            // If the previous key pressed was not an operation
            // perform the previous operation
            if (!self.isLastActionOperation || operation == Operation.Equals) {
                
                var operationPerformed = true
                var result = 0.0
                
                switch doOperation {
                    
                case .Divide:
                    if self.isLastActionOperation {
                        result = self.divide(currentValue, by: self.previousValue)
                    }
                    else {
                        result = self.divide(self.previousValue, by: currentValue)
                    }
                case .Multiply:
                    result = self.multiply(self.previousValue, times: currentValue)
                    
                case .Add:
                    result = self.add(self.previousValue, plus: currentValue)
                    
                case .Subtract:
                    if self.isLastActionOperation {
                        result = self.subtract(currentValue, minus: self.previousValue)
                    }
                    else {
                        result = self.subtract(self.previousValue, minus: currentValue)
                    }
                default:
                    operationPerformed = false
                }
                
                // If a basic operation was performed, store the
                // result in the current value
                if operationPerformed {
                    var number = NSNumber(double: result)
                    self.value = self.formatter.stringFromNumber(number)!
                    self.previousOperationPerformed = doOperation
                }
                
                // The following statement allows the calculator to perform
                // the typical functionality where pressing the equal sign
                // multiple times repeats the previous operation
                
                // If the current operation is not Equals, store the
                // current value in theh previous value
                if operation != Operation.Equals {
                    
                    self.previousValue = (self.value.stringByReplacingOccurrencesOfString(",", withString: "") as NSString).doubleValue
                }
                else if self.previousOperation != Operation.Equals {
                    
                    // The current operation is Equals, but the previous operation
                    // is not. Store the current value in the previous value
                    self.previousValue = result
                }
            }
            
            // Store the current operation as the previous operation
            self.previousOperation = operation
        }
        
        self.isLastActionOperation = true
        return self.value
    }
    
    // Clears the current and previous values
    func clear() {
        
        self.value = "0"
        self.previousValue = 0
        self.isLastActionOperation = false
        self.previousOperation = Operation.None
        self.previousOperationPerformed = Operation.None
        self.hideOperationHighlight()
    }
    
    // Adds two values and returns the result
    func add(value1: Double, plus value2: Double) -> Double {
        return value1 + value2
    }
    
    // Subtracts two values and returns the result
    func subtract(value1: Double, minus value2: Double) -> Double
    {
        return value1 - value2
    }
    
    // Multiplies two values and returns the result
    func multiply(value1: Double, times value2: Double) -> Double
    {
        return value1 * value2
    }
    
    // Divides two values and returns the results
    func divide(dividend: Double, by divisor: Double) -> Double
    {
        var error: NSError?
        if divisor == 0  {
            // Can't divide by zero. This is an error
            NSException.raise("DivideByZero", format: "Cannot divide by zero", arguments:getVaList([error!]))
            
        }
        return dividend / divisor
    }
    
    // Makes a postive number negative and a negative number positive
    func makePositiveNegative() {
        // Convert the current string value to double, then multiply times -1
        var valueDouble = (self.value.stringByReplacingOccurrencesOfString(",", withString: "") as NSString).doubleValue
        
        valueDouble = valueDouble * -1
        
        self.previousValue = valueDouble
        
        // Convert new value to a formatted string
        self.value = self.formatter.stringFromNumber(NSNumber(double:valueDouble))!
    }
    
    func applyPercentage(value: Double) -> Double {
        
        var tempValue = value
        if (self.previousOperation == .Add || self.previousOperation == .Subtract) {
            // current value = (current value *.01) * previous value
            tempValue = (value * 0.01) * self.previousValue
        }
        else
        {
            // current value = current value *.01
            tempValue = (value * 0.01)
        }
        return tempValue
    }
    
    
}



